@@ -12,6 +12,9 @@ import ddb_single.utils_botos as util_b
 def default_pk_factory(model_name):
     return f"{model_name}_{uuid.uuid4().hex}"
 
+def default_sk_factory(model_name, prefix="", suffix="_item"):
+    return f"{prefix}{model_name}{suffix}"
+
 def default_pk2model(pk):
     return pk.split("_", 1)[0]
 
@@ -55,6 +58,7 @@ class Table:
         secondary_key_type="S", 
         secondary_key_prefix="", 
         secondary_key_suffix="_item",
+        secondary_key_factory=default_sk_factory,
         unique_key="unique", 
         unique_key_type="S",
         search_prefix="search_", 
@@ -83,6 +87,7 @@ class Table:
         self.__secondary_key_type__ = secondary_key_type
         self.__secondary_key_prefix__ = secondary_key_prefix
         self.__secondary_key_suffix__ = secondary_key_suffix
+        self.__secondary_key_factory__ = secondary_key_factory
         self.__unique_key__ = unique_key
         self.__unique_key_type__ = unique_key_type
         self.__search_prefix__ = search_prefix
@@ -104,7 +109,7 @@ class Table:
         return self.__primary_key2model__(pk)
 
     def sk(self, model_name):
-        return f"{self.__secondary_key_prefix__}{model_name}{self.__secondary_key_suffix__}"
+        return self.__secondary_key_factory__(model_name, self.__secondary_key_prefix__, self.__secondary_key_suffix__)
     
     def sk2model(self, sk):
         return sk[len(self.__secondary_key_prefix__):-len(self.__secondary_key_suffix__)]
@@ -166,7 +171,7 @@ class Table:
                     print(f"{len(res_data)}/{limit}")
                     response = self.__table__.scan(**kwargs, ExclusiveStartKey=response['LastEvaluatedKey'])
                     res_data += response['Items']
-                return json_export(res_data)
+                return util_b.json_export(res_data)
             else:
                 return []
 
@@ -253,27 +258,48 @@ class Table:
 
     # --- 検索関連 ---
     # 関連先を検索
-    def relation(self, pk, model_name=""):
+    def relation(self, pk, model_name="", field_name="", pk_only=False):
         KeyConditionExpression = Key(self.__primary_key__).eq(pk)
         if model_name:
             KeyConditionExpression &= Key(self.__secondary_key__).begins_with(self.rel_prefix(model_name))
-        res = self._query(
-            KeyConditionExpression=KeyConditionExpression
-        )
-        res = self.batch_get_from_pks([r[self.__primary_key__] for r in res])
-        return res
+        if field_name:
+            # フィールドの指定がある場合
+            res = self._query(
+                KeyConditionExpression=KeyConditionExpression,
+                FilterExpression=Attr(self.__search_data_key__).eq(field_name),
+            )
+        else:
+            # モデルの指定がある場合
+            res = self._query(
+                KeyConditionExpression=KeyConditionExpression
+            )
+        pks = [self.rel_key2pk(r[self.__secondary_key__]) for r in res]
+        if pk_only:
+            return pks
+        else:
+            return self.batch_get_from_pks(pks)
 
     # 関連元を検索
-    def reference(self, pk, model_name=""):
+    def reference(self, pk, model_name="", field_name="", pk_only=False):
         KeyConditionExpression = Key(self.__secondary_key__).eq(self.rel_key(pk))
-        if model_name:
-            KeyConditionExpression &= Key(self.__primary_key__).begins_with(model_name)
-        res = self._query(
-            KeyConditionExpression=KeyConditionExpression,
-            IndexName=self.__range_index_name__,
-        )
-        res = self.batch_get_from_pks([r[self.__primary_key__] for r in res])
-        return res
+        if field_name:
+            # フィールドの指定がある場合
+            res = self._query(
+                KeyConditionExpression=KeyConditionExpression & Key(self.__search_data_key__).eq(field_name),
+                FilterExpression=Attr(self.__primary_key__).begins_with(model_name),
+                IndexName=self.__search_index__
+            )
+        else:
+            # モデルの指定がある場合
+            res = self._query(
+                KeyConditionExpression=KeyConditionExpression & Key(self.__primary_key__).begins_with(model_name),
+                IndexName=self.__range_index_name__,
+            )
+        pks = [r[self.__primary_key__] for r in res]
+        if pk_only:
+            return pks
+        else:
+            return self.batch_get_from_pks(pks)
     
     # フィルター
     def filter(self, items, searchExs):
@@ -287,7 +313,7 @@ class Table:
         return res
 
     # 検索
-    def search(self, model_name, *searchEx):
+    def search(self, model_name, *searchEx, pk_only=False):
         simple_ex = [s for s in searchEx if s["FilterStatus"] == util_b.FilterStatus.SEATCH]
         staged_ex = [s for s in searchEx if s["FilterStatus"] == util_b.FilterStatus.STAGED]
         filter_ex = [s for s in searchEx if s["FilterStatus"] == util_b.FilterStatus.FILTER]
@@ -305,9 +331,6 @@ class Table:
                     res &= set(_res)
                 else:
                     res = set(_res)
-            res = self.batch_get_from_pks(list(res))
-            #  filter_ex があればフィルタ
-            return self.filter(res, filter_ex)
         else:
             # シンプルにクエリ検索
             KeyConditionExpression = Key(self.__secondary_key__).eq(self.sk(model_name))
@@ -326,8 +349,16 @@ class Table:
                 res = self._query(
                     KeyConditionExpression=KeyConditionExpression
                 ) or []
-            # staged_ex があればフィルタ
-            return self.filter(res, staged_ex)
+        if pk_only and not filter_ex:
+            return list(res)
+        else:
+            res = self.batch_get_from_pks(list(res))
+            #  filter_ex があればフィルタ
+            res = self.filter(res, filter_ex)
+            if pk_only:
+                return [r[self.__primary_key__] for r in res]
+            else:
+                return res
 
     # --- 更新関連 ---
     # バッチライター
